# Generated by Django 4.0.4 on 2022-07-05 23:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_dans_notifications", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="notificationemailtemplate",
            options={"ordering": ["datetime_created"]},
        ),
    ]



import os
from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = "secret"

DEBUG = True

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "django-dans-notifications-db",
        "USER": "django-dans-notifications",
        "PASSWORD": "test-password",
    }
}

MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

# New style default middleware declaration for Django 1.10+
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.admin",
    "django.contrib.messages",
    # rest framework
    "rest_framework",
    "rest_framework.authtoken",
    "django_dans_notifications",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(BASE_DIR, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]


# This is for Django 4.0, harmless for previous versions.
USE_TZ = False

# This is for Django 3.2, harmless for previous versions.
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"


REST_FRAMEWORK = {
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.PageNumberPagination",
    "PAGE_SIZE": 20,
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework.authentication.TokenAuthentication",
    ),
}


IN_TEST = True



import json
import uuid

from .base import BaseAPITestCase
from ...models.push import NotificationPush
from ...views.push import NotificationPushViewSet

"""
# =============================================================================================
# TEST NOTIFICATION PUSH VIEW SET =============================================================
# =============================================================================================
"""


class TestNotificationPushViewSet(BaseAPITestCase):
    def setUp(self):
        super(TestNotificationPushViewSet, self).setUp()
        self.view_list = NotificationPushViewSet.as_view({"get": "list"})
        self.view_retrieve = NotificationPushViewSet.as_view({"get": "retrieve"})

    @staticmethod
    def get_url():
        return "/api/notifications/push/"

    @staticmethod
    def get_url_pk(pk):
        return f"/api/notifications/push/{pk}/"

    # ==================================================================================
    # GET - LIST =======================================================================
    # ==================================================================================

    def test_notifications_push_list_none(self):
        # make api request
        request = self.factory.get(
            self.get_url(), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_list(request)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["count"], 0)
        self.assertEqual(json_response["next"], None)
        self.assertEqual(json_response["previous"], None)
        self.assertEqual(json_response["results"], [])

    def test_notification_push_list_one_not_recp(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create()

        # make api request
        request = self.factory.get(
            self.get_url(), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_list(request)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["count"], 0)
        self.assertEqual(json_response["next"], None)
        self.assertEqual(json_response["previous"], None)
        self.assertEqual(json_response["results"], [])

    def test_notification_push_list_one(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create(recipients=self.email)

        # make api request
        request = self.factory.get(
            self.get_url(), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_list(request)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["count"], 1)
        self.assertEqual(json_response["next"], None)
        self.assertEqual(json_response["previous"], None)

    def test_notification_push_list_man_not_recp(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create()
        notification2 = NotificationPush.objects.create()
        notification3 = NotificationPush.objects.create()

        # make api request
        request = self.factory.get(
            self.get_url(), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_list(request)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["count"], 0)
        self.assertEqual(json_response["next"], None)
        self.assertEqual(json_response["previous"], None)
        self.assertEqual(json_response["results"], [])

    def test_notification_push_list_man_one_recp(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create(recipients=self.email)
        notification2 = NotificationPush.objects.create()
        notification3 = NotificationPush.objects.create()

        # make api request
        request = self.factory.get(
            self.get_url(), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_list(request)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["count"], 1)
        self.assertEqual(json_response["next"], None)
        self.assertEqual(json_response["previous"], None)

    def test_notification_push_list_man_three_recp(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create(recipients=self.email)
        notification2 = NotificationPush.objects.create(recipients=self.email)
        notification3 = NotificationPush.objects.create(recipients=self.email)

        # make api request
        request = self.factory.get(
            self.get_url(), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_list(request)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["count"], 3)
        self.assertEqual(json_response["next"], None)
        self.assertEqual(json_response["previous"], None)

    # ==================================================================================
    # GET - RETRIEVE ===================================================================
    # ==================================================================================

    def test_notification_push_retrieve_pk_valid_not_recp(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create()
        pk = notification1.pk

        # make api request
        request = self.factory.get(
            self.get_url_pk(pk=pk), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_retrieve(request, pk=pk)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 400)
        self.assertEqual(json_response["error"], "Notification not found.")

    def test_notification_push_retrieve_pk_valid(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create(recipients=self.email)
        pk = notification1.pk

        # make api request
        request = self.factory.get(
            self.get_url_pk(pk=pk), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_retrieve(request, pk=pk)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["success"]["id"], str(pk))

    def test_notification_push_retrieve_pk_valid_multiple_notification_push_not_recp(
        self,
    ):
        # create notification(s)
        notification1 = NotificationPush.objects.create()
        notification2 = NotificationPush.objects.create()
        notification3 = NotificationPush.objects.create()
        pk = notification1.pk

        # make api request
        request = self.factory.get(
            self.get_url_pk(pk=pk), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_retrieve(request, pk=pk)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 400)
        self.assertEqual(json_response["error"], "Notification not found.")

    def test_notification_push_retrieve_pk_valid_multiple_notification_push(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create(recipients=self.email)
        notification2 = NotificationPush.objects.create()
        notification3 = NotificationPush.objects.create()
        pk = notification1.pk

        # make api request
        request = self.factory.get(
            self.get_url_pk(pk=pk), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_retrieve(request, pk=pk)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 200)
        self.assertEqual(json_response["success"]["id"], str(pk))

    def test_notification_push_retrieve_pk_invalid_uuid(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create()
        notification2 = NotificationPush.objects.create()
        notification3 = NotificationPush.objects.create()
        pk = "invalid"

        # make api request
        request = self.factory.get(
            self.get_url_pk(pk=pk), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_retrieve(request, pk=pk)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 400)
        self.assertEqual(json_response["error"], "Notification not found.")

    def test_notification_push_retrieve_pk_missing(self):
        # create notification(s)
        notification1 = NotificationPush.objects.create()
        notification2 = NotificationPush.objects.create()
        notification3 = NotificationPush.objects.create()
        pk = uuid.uuid4()

        # make api request
        request = self.factory.get(
            self.get_url_pk(pk=pk), HTTP_AUTHORIZATION=f"Token {self.user_token}"
        )
        response = self.view_retrieve(request, pk=pk)
        response.render()
        json_response = json.loads(response.content)

        # confirm status code and data
        self.assertEqual(response.status_code, 400)
        self.assertEqual(json_response["error"], "Notification not found.")



from django.contrib import admin

from .models.basic import NotificationBasic
from .models.email import NotificationEmail, NotificationEmailTemplate
from .models.push import NotificationPush

"""
# ==================================================================================== #
# NOTIFICATIONS ====================================================================== #
# ==================================================================================== #
"""


# ====================================================== #
# NOTIFICATION BASIC =================================== #
# ====================================================== #


class NotificationBasicAdmin(admin.ModelAdmin):
    list_display = (
        "sender",
        "recipients",
        "read",
        "message",
        "datetime_created",
        "datetime_sent",
        "sent_successfully",
    )
    list_display_links = ("sender", "message")
    search_fields = (
        "datetime_sent",
        "sent_successfully",
        "recipients",
        "sender",
        "message",
    )
    list_per_page = 100


# ====================================================== #
# NOTIFICATION EMAIL =================================== #
# ====================================================== #


class NotificationEmailTemplateAdmin(admin.ModelAdmin):
    list_display = (
        "nickname",
        "path",
    )
    list_display_links = ("nickname",)
    search_fields = (
        "nickname",
        "path",
    )
    list_per_page = 100


class NotificationEmailAdmin(admin.ModelAdmin):
    list_display = (
        "sender",
        "recipients",
        "subject",
        "template",
        "datetime_created",
        "datetime_sent",
        "sent_successfully",
    )
    list_display_links = ("sender", "subject")
    search_fields = (
        "datetime_sent",
        "sent_successfully",
        "recipients",
        "sender",
        "template",
    )
    list_per_page = 100


# ====================================================== #
# NOTIFICATION PUSH ==================================== #
# ====================================================== #


class NotificationPushAdmin(admin.ModelAdmin):
    list_display = (
        "sender",
        "recipients",
        "message",
        "datetime_created",
        "datetime_sent",
        "sent_successfully",
    )
    list_display_links = ("sender", "message")
    search_fields = (
        "datetime_sent",
        "sent_successfully",
        "recipients",
        "sender",
        "message",
    )
    list_per_page = 100


"""
# ==================================================================================== #
# REGISTER =========================================================================== #
# ==================================================================================== #
"""

admin.site.register(NotificationBasic, NotificationBasicAdmin)
admin.site.register(NotificationEmailTemplate, NotificationEmailTemplateAdmin)
admin.site.register(NotificationEmail, NotificationEmailAdmin)
admin.site.register(NotificationPush, NotificationPushAdmin)



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.template import TemplateDoesNotExist
from django.template.loader import render_to_string, get_template
from django.utils import timezone
from django.utils.html import strip_tags
from smtplib import SMTPException

from .base import NotificationBase, AbstractBaseModel


"""
# ==================================================================================== #
# NOTIFICATION EMAIL ================================================================= #
# ==================================================================================== #
"""


#
# NOTIFICATION EMAIL TEMPLATE MANAGER ================== #
#
class NotificationEmailTemplateManager(models.Manager):
    """
    NotificationEmailTemplateManager

    Manager for NotificationEmailTemplate.
    """

    use_in_migrations = True

    @staticmethod
    def find_email_template(template):
        """
        get or create NotificationEmailTemplate object for the passed 'template'

        :param str template: Path of email template to use should be of this form 'django-dans-emails/<NAME>.html'. Can also be the templates 'nickname'.
        """
        try:
            return NotificationEmailTemplate.objects.get(path=template)
        except NotificationEmailTemplate.DoesNotExist:
            pass

        # see if file exists at path 'emails/'
        try:
            return NotificationEmailTemplate.objects.get(
                path=f"django-dans-emails/{template}"
            )
        except NotificationEmailTemplate.DoesNotExist:
            pass

        # see if file exists at path 'emails/'
        try:
            return NotificationEmailTemplate.objects.get(path=f"emails/{template}")
        except NotificationEmailTemplate.DoesNotExist:
            pass

        # path doesn't exist - see if 'template' is a nickname
        try:
            email_templates = NotificationEmailTemplate.objects.filter(
                nickname=template
            )
            if email_templates.count() > 0:
                return email_templates.first()
        except NotificationEmailTemplate.DoesNotExist:
            pass

        # template not found by path or nickname - see if file exists or not
        try:
            tmp = get_template(template)
            return NotificationEmailTemplate.objects.get_or_create(
                path=template, nickname=template
            )[0]
        except TemplateDoesNotExist:
            pass

        # see if file exists at "django-dans-emails/"
        try:
            tmp = get_template(f"django-dans-emails/{template}")
            return NotificationEmailTemplate.objects.get_or_create(
                path=template, nickname=template
            )[0]
        except TemplateDoesNotExist:
            pass

        # see if file exists at "emails/"
        try:
            tmp = get_template(f"emails/{template}")
            return NotificationEmailTemplate.objects.get_or_create(
                path=template, nickname=template
            )[0]
        except TemplateDoesNotExist:
            pass

        return None


#
# NOTIFICATION EMAIL MANAGER ================== #
#
class NotificationEmailManager(models.Manager):
    """
    NotificationEmailManager

    Manager for NotificationEmail.

    Handles sending and management of objects.
    """

    use_in_migrations = True

    @staticmethod
    def send_email(
        subject="Email from Dan's Backend",
        template="django-dans-emails/default.html",
        sender=settings.DEFAULT_FROM_EMAIL,
        recipients=None,
        context=None,
        file_attachment=None,
    ):
        """
        Send email function - sends email, handles notification system and object creation and everything
        WILL NOT send in test mode - set via 'IN_TEST' in settings.py file.

        :param str subject: Subject for email.
        :param str template: Template file path or nickname.
        :param str sender: From email.
        :param str recipients: List of email(s) to send to.
        :param dict context: Context dict for template.
        :param file_attachment: File attachment to send with email.
        """
        if recipients is None:
            recipients = ""

        # check if model exists for the .html file at the given path
        # if so, ensure that an EmailTemplate objects exists for it
        # if not, error
        email_template = NotificationEmailTemplate.objects.find_email_template(
            template=template
        )
        if not email_template:
            raise ValueError("EmailTemplate with path/nickname does not exist.")

        # add TEAM_NAME var to context if appropriate
        if hasattr(settings, "TEAM_NAME") and context:
            if "team_name" not in context:
                context["team_name"] = settings.TEAM_NAME

        # render html with context object
        html_string = email_template.html_to_str(context)

        # create EmailNotification object
        notification_email = NotificationEmail.objects.create(
            template=email_template,
            subject=subject,
            context=context,
            sender=sender,
            recipients=recipients,
        )

        # send email via django
        try:
            text_content = strip_tags(html_string)
            message = EmailMultiAlternatives(
                subject=subject,
                body=text_content,
                from_email=sender,
                to=notification_email.recipients_list,
            )
            message.attach_alternative(html_string, "text/html")
            try:
                if file_attachment is not None:  # attach file if applicable
                    message.attach(file_attachment.name, file_attachment.read())
            except (AttributeError, Exception) as e:  # TODO catch specific exception
                print(f"Issue attaching to email: {type(e)} - {e}")

            if hasattr(settings, "IN_TEST") and settings.IN_TEST:
                pass  # dont send mail in tests
            else:
                message.send(fail_silently=False)
            notification_email.sent_successfully = True
        except (
            SMTPException,
            Exception,
        ) as e:  # TODO - could potentially throw other exceptions, catch specific ones
            print(f"Error creating and sending email: {type(e)} - {e}")
            notification_email.sent_successfully = False
        notification_email.datetime_sent = timezone.now()  # save regardless of status
        notification_email.save()
        return notification_email


#
# NOTIFICATION EMAIL TEMPLATE =============== #
#
class NotificationEmailTemplate(AbstractBaseModel):
    objects = NotificationEmailTemplateManager()

    path = models.CharField(max_length=300, null=False, blank=False)
    nickname = models.CharField(max_length=300, null=False, blank=False)

    def __str__(self):
        return "Email Template: " + str(self.nickname)

    def html_to_str(self, context):
        try:
            return render_to_string(self.path, context)
        except Exception as e:
            print(f"Error rendering email template: ({type(e)}) {e}")
            return render_to_string("django-dans-emails/default.html", context)


def get_default_template():
    return NotificationEmailTemplate.objects.get_or_create(
        path="django-dans-emails/default.html"
    )[0]


#
# NOTIFICATION EMAIL =================== #
#
class NotificationEmail(NotificationBase):
    objects = NotificationEmailManager()

    template = models.ForeignKey(
        NotificationEmailTemplate,
        related_name="template",
        default=get_default_template,
        on_delete=models.DO_NOTHING,
    )
    subject = models.CharField(max_length=300, null=False, blank=False)
    context = models.JSONField(null=True, blank=True)

    def __str__(self):
        return f"Notification Email: {self.sender} -> {self.recipients}"



import uuid

from django.db import models

"""
# ==================================================================================== #
# ABSTRACT BASE MODEL ================================================================ #
# ==================================================================================== #
"""


class AbstractBaseModel(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    datetime_created = models.DateTimeField(auto_now_add=True, editable=False)
    datetime_modified = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True
        ordering = ["datetime_created"]

    def __str__(self):
        return "Abstract Base Model"


"""
# ==================================================================================== #
# NOTIFICATION BASE ================================================================== #
# ==================================================================================== #
"""


#
# NOTIFICATION BASE =================== #
#
class NotificationBase(AbstractBaseModel):
    datetime_sent = models.DateTimeField(null=True, blank=True)
    sent_successfully = models.BooleanField(default=False, null=False, blank=False)
    sender = models.CharField(
        max_length=300,
        null=False,
        blank=False,
        help_text="This should be the sending users email.",
    )
    recipients = models.CharField(
        max_length=900,
        null=False,
        blank=False,
        help_text="Comma separated list of email recipients.",
    )

    class Meta:
        abstract = True

    def __str__(self):
        return "Notification Base"

    def save(self, keep_deleted=False, **kwargs):
        # cleanup 'recipients'
        self.recipients = self.recipients_cleanup()
        return super(NotificationBase, self).save(**kwargs)

    @property
    def recipients_list(self):
        return self.recipients.split(",")

    def recipients_cleanup(self):
        # take in whatever is currently set as recipients
        list_recipients = self.recipients

        # if a str or conversion of a list, convert to a list of the recipients
        if "," in list_recipients or "[" in list_recipients or "]" in list_recipients:
            tmp = (
                str(self.recipients).replace("[", "").replace("]", "").replace(" ", "")
            )
            list_recipients = tmp.split(",")

        # if the last element is empty, remove it
        if len(list_recipients) > 0:
            if list_recipients[len(list_recipients) - 1] == "":
                list_recipients.pop()

        # if a list, convert standardize as str 'recp1,recp2,recp3'
        if type(list_recipients) == list:
            res = ",".join(str(e) for e in list_recipients)
        else:  # probably a string, just return as is
            res = list_recipients

        # remove any extra chars
        res = res.replace("'", "").replace('"', "")
        return res

    def recipients_contains(self, user):
        """
        Detect if 'user' is involved with this notification or not
        """
        if str(user) in self.recipients:
            return True
        return False
